@@ -3,7 +3,7 @@
 #include "cfvector.h"
 #include "chebyshev.h"
 #include "flowfield.h"
-#include "nsintegrator.h"
+#include "dns.h"
 
 int main() {
 
@@ -25,7 +25,7 @@ int main() {
   // Define integration parameters
   const int n = 20;     // take ten steps between printouts
   const Real dt = 0.05;
-  const Real T  = 100.0;
+  const Real T  = 10.0;
 
   // Define size and smoothness of initial disturbance
   Real decay = 0.6;   
@@ -50,10 +50,10 @@ int main() {
     
   // Construct Navier-Stoke Integrator
   DNSFlags flags; 
-  flags.timestepping = RK3;             // use 3rd-order Runge-Kutta method
+  flags.timestepping = CNAB2;             // use 3rd-order Runge-Kutta method
   flags.constraint  = PressureGradient; // enforce constant pressure gradient
 
-  NSIntegrator dns(u, U, nu, dt, flags);
+  DNS dns(u, U, nu, dt, flags);
 
   for (Real t=0; t<T; t += n*dt) {
     cout << "===============================================" << endl;
