@@ -1,13 +1,16 @@
 /* basisfunc.cpp: a Complex-Vector-valued Spectral expansion class.
  * Channelflow-0.9
  *
- * Copyright (C) 2001  John F. Gibson  
- *  
- * jgibson@mail.sjcsf.edu  
- * John F. Gibson 
- * St. John's College
- * 1160 Camino de la Cruz Blanca
- * Santa Fe, NM 87501
+ * Copyright (C) 2001-2005  John F. Gibson
+ *
+ * John F. Gibson
+ * Center for Nonlinear Sciences
+ * School of Physics
+ * Georgia Institute of Technology
+ * Atlanta, GA 30332-0430
+ *
+ * gibson@cns.physics.gatech.edu
+ * jfg@member.fsf.org
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -175,40 +178,46 @@ BasisFunc::BasisFunc(const ComplexChebyCoeff& u,
   assert(u.congruent(v) && u.congruent(w));
 }
 
-void BasisFunc::save(const string& filebase) const {
+void BasisFunc::save(const string& filebase, fieldstate savestate) const {
+  fieldstate origstate = state_;
+  ((BasisFunc&) *this).makeState(savestate);
+
   string filename(filebase);
   filename += string(".bf");
   ofstream os(filename.c_str());
-  os << setprecision(REAL_DIGITS);
-  char s=' ';
+  os << scientific << setprecision(REAL_DIGITS);
+  char sp=' ';
   os << '%'
-     <<s<< Ny_
-     <<s<< kx_
-     <<s<< kz_
-     <<s<< Lx_
-     <<s<< Lz_
-     <<s<< a_
-     <<s<< b_
-     <<s<< state_ 
-     <<s<< lambda_
+     <<sp<< Ny_
+     <<sp<< kx_
+     <<sp<< kz_
+     <<sp<< Lx_
+     <<sp<< Lz_
+     <<sp<< a_
+     <<sp<< b_
+     <<sp<< state_ 
+     <<sp<< lambda_
      << '\n';
 
   for (int i=0; i<Ny_; ++i) 
-    os << Re(u_[i]) <<s<< Im(u_[i]) <<s
-       << Re(v_[i]) <<s<< Im(v_[i]) <<s
-       << Re(w_[i]) <<s<< Im(w_[i]) <<'\n';
+    os << Re(u_[i]) <<sp<< Im(u_[i]) <<sp
+       << Re(v_[i]) <<sp<< Im(v_[i]) <<sp
+       << Re(w_[i]) <<sp<< Im(w_[i]) <<'\n';
   os.close();
+
+  ((BasisFunc&) *this).makeState(origstate);
 }
 
 void BasisFunc::m4save(const string& filebase) const {
   string filename = filebase + string(".asc");
   ofstream os(filename.c_str());
   os << setprecision(REAL_DIGITS);
-  char s=' ';
+  const int w = REAL_IOWIDTH;
+  const char s=' ';
   for (int i=0; i<Ny_; ++i) 
-    os << Re(u_[i]) <<s<< Im(u_[i]) <<s
-       << Re(v_[i]) <<s<< Im(v_[i]) <<s
-       << Re(w_[i]) <<s<< Im(w_[i]) <<'\n';
+    os << setw(w) << Re(u_[i]) <<s<< setw(w) << Im(u_[i]) <<s
+       << setw(w) << Re(v_[i]) <<s<< setw(w) << Im(v_[i]) <<s
+       << setw(w) << Re(w_[i]) <<s<< setw(w) << Im(w_[i]) <<'\n';
   os.close();
 }
 
@@ -370,9 +379,9 @@ void BasisFunc::conjugate() {
 Real BasisFunc::bcNorm(bool a_dirichlet, bool b_dirichlet) const {
   Real err=0.0;
   if (b_dirichlet) 
-    err += norm2(u_.eval_b()) + norm2(v_.eval_b()) + norm2(w_.eval_b());
+    err += abs2(u_.eval_b()) + abs2(v_.eval_b()) + abs2(w_.eval_b());
   if (a_dirichlet)
-    err += norm2(u_.eval_a()) + norm2(v_.eval_a()) + norm2(w_.eval_a());
+    err += abs2(u_.eval_a()) + abs2(v_.eval_a()) + abs2(w_.eval_a());
   return sqrt(err);
 }
 
@@ -751,9 +760,9 @@ Real divDist(const BasisFunc& f, const BasisFunc& g, bool normalize)  {
 
 Real bcNorm2(const BasisFunc& f, bool normalize) {
   Real bc2 = 
-    norm2(f.u().eval_a()) + norm2(f.u().eval_b()) +
-    norm2(f.v().eval_a()) + norm2(f.v().eval_b()) +
-    norm2(f.w().eval_a()) + norm2(f.w().eval_b());
+    abs2(f.u().eval_a()) + abs2(f.u().eval_b()) +
+    abs2(f.v().eval_a()) + abs2(f.v().eval_b()) +
+    abs2(f.w().eval_a()) + abs2(f.w().eval_b());
   if (!normalize)
     bc2 *= f.Lx()*f.Lz();
   return bc2;
@@ -765,12 +774,12 @@ Real bcNorm(const BasisFunc& f, bool normalize)  {
 
 Real bcDist2(const BasisFunc& f, const BasisFunc& g, bool normalize) {
   Real bc2 = 
-    norm2(f.u().eval_a() - g.u().eval_a()) + 
-    norm2(f.u().eval_b() - g.u().eval_b()) + 
-    norm2(f.v().eval_a() - g.v().eval_a()) + 
-    norm2(f.v().eval_b() - g.v().eval_b()) + 
-    norm2(f.w().eval_a() - g.w().eval_a()) + 
-    norm2(f.w().eval_b() - g.w().eval_b());
+    abs2(f.u().eval_a() - g.u().eval_a()) + 
+    abs2(f.u().eval_b() - g.u().eval_b()) + 
+    abs2(f.v().eval_a() - g.v().eval_a()) + 
+    abs2(f.v().eval_b() - g.v().eval_b()) + 
+    abs2(f.w().eval_a() - g.w().eval_a()) + 
+    abs2(f.w().eval_b() - g.w().eval_b());
   if (!normalize)
     bc2 *= f.Lx()*f.Lz();
   return bc2;
