@@ -1,13 +1,16 @@
 /* basisfunc.h: a Complex-Vector-valued Chebyshev expansion class.
  * Channelflow-0.9
  *
- * Copyright (C) 2001  John F. Gibson  
- *  
- * jgibson@mail.sjcsf.edu  
- * John F. Gibson 
- * St. John's College
- * 1160 Camino de la Cruz Blanca
- * Santa Fe, NM 87501
+ * Copyright (C) 2001-2005  John F. Gibson
+ *
+ * John F. Gibson
+ * Center for Nonlinear Sciences
+ * School of Physics
+ * Georgia Institute of Technology
+ * Atlanta, GA 30332-0430
+ *
+ * gibson@cns.physics.gatech.edu
+ * jfg@member.fsf.org
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -42,7 +45,7 @@ public:
 	    const ComplexChebyCoeff& w,
 	    Real lambda, int kx, int kz, Real Lx, Real Lz);
 
-  void save(const string& filebase) const; 
+  void save(const string& filebase, fieldstate s=Physical) const; 
   void m4save(const string& filebase) const; 
   void binaryDump(ostream& os) const; 
   void binaryLoad(istream& is); 
@@ -96,7 +99,7 @@ public:
   ComplexChebyCoeff& operator[](int i);
 
   Real bcNorm(bool a_dirichlet=true, bool b_dirichlet=true) const;  
-  Real divNorm() const; 
+  //Real divNorm() const; 
   bool geometryCongruent(const BasisFunc& f) const;
   bool congruent(const BasisFunc& f) const; // geomCongruent & kx,kz equality
   bool operator==(const BasisFunc& f) const;
@@ -125,6 +128,23 @@ protected:
   ComplexChebyCoeff w_;
 };
 
+// L2Norm2(f)  == Int ||f||^2     dx dy dz (/(Lx Ly Lz) if normalize==true)
+// bcNorm2(f)  == Int ||f||^2     dx dz at y=a,b (/(Lx Lz) if normalize==true)
+// divNorm2(f) == Int ||div f||^2 dx dy dz (/(Lx Ly Lz) if normalize==true)
+
+// innerProduct(f,g) == Int f g*  dx dy dz (/(Lx Ly Lz) if normalize==true)
+
+// L2Norm(f)    == sqrt(L2Norm2(f))
+// L2Dist(f,g)  == sqrt(L2Dist2(f,g))
+// L2Dist2(f,g) == L2Norm2(f-g)
+// etc.
+
+// These are not so simple to compute for individual Fourier modes.X
+//Real L1Norm(const BasisFunc& f, bool normalize=true);
+//Real L1Dist(const BasisFunc& f, const BasisFunc& g, bool normalize=true);
+//Real LinfNorm(const BasisFunc& f);
+//Real LinfDist(const BasisFunc& f, const BasisFunc& g);
+
 Real L2Norm(const BasisFunc& f, bool normalize=true);
 Real L2Norm2(const BasisFunc& f, bool normalize=true);
 Real L2Dist(const BasisFunc& f, const BasisFunc& g, bool normalize=true);
@@ -146,6 +166,16 @@ Real dist2(const BasisFunc& f, const BasisFunc& g, NormType n,bool nrmlz=true);
 Complex innerProduct(const BasisFunc& f0, const BasisFunc& f1, 
 		     NormType n, bool normalize=true);
 
+Real bcNorm(const BasisFunc& f, bool normalize=true);
+Real bcNorm2(const BasisFunc& f, bool normalize=true);
+Real bcDist(const BasisFunc& f, const BasisFunc& g, bool normalize=true);
+Real bcDist2(const BasisFunc& f, const BasisFunc& g, bool normalize=true);
+
+Real divNorm(const BasisFunc& f, bool normalize=true);
+Real divNorm2(const BasisFunc& f, bool normalize=true);
+Real divDist(const BasisFunc& f, const BasisFunc& g, bool normalize=true);
+Real divDist2(const BasisFunc& f, const BasisFunc& g, bool normalize=true);
+
 BasisFunc xdiff(const BasisFunc& f);
 BasisFunc ydiff(const BasisFunc& f);
 BasisFunc zdiff(const BasisFunc& f);
