@@ -3,11 +3,13 @@
  *
  * Copyright (C) 2001  John F. Gibson  
  *  
- * jgibson@mail.sjcsf.edu  
+ * Copyright (C) 2001-05  John F. Gibson  
+ *  
  * John F. Gibson 
- * St. John's College
- * 1160 Camino de la Cruz Blanca
- * Santa Fe, NM 87501
+ * Center for Nonlinear Sciences
+ * School of Physics
+ * Georgia Institute of Technology
+ * Atlanta, GA  GA 30332-0430
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -430,29 +432,21 @@ Real ChebyCoeff::mean() const {
 
 //Real ChebyCoeff::energy() const {return L2Norm2(*this, false);}
 
-void ChebyCoeff::save(const string& filebase) const {
+void ChebyCoeff::save(const string& filebase, fieldstate savestate) const {
+  fieldstate origstate = state_;
+  ((ChebyCoeff&) *this).makeState(savestate);
+
   string filename(filebase);
   filename += string(".asc");
   ofstream os(filename.c_str());
-  os << setprecision(REAL_DIGITS);
+  os << scientific << setprecision(REAL_DIGITS);
   os << "% " << N_ << ' ' << a_ << ' ' << b_ << ' ' << state_ << '\n';
   for (int i=0; i<N_; ++i) 
-    os << data_[i] << '\n';
+    os << setw(REAL_IOWIDTH) << data_[i] << '\n';
   os.close();
+  ((ChebyCoeff&) *this).makeState(origstate);
 }
 
-/**************
-void ChebyCoeff::m4save(const string& filebase) const {
-  string filename(filebase);
-  filename += string(".asc");
-  ofstream os(filename.c_str());
-  os << setprecision(REAL_DIGITS);
-  for (int i=0; i<N_; ++i) 
-    os << data_[i] << '\n';
-  os.close();
-}
-******************/
-
 void ChebyCoeff::binaryDump(ostream& os) const {
   write(os, N_);
   write(os, a_);
@@ -794,6 +788,80 @@ Real chebyInnerProduct(const ChebyCoeff& u, const ChebyCoeff& v, bool normalize)
   return sum*pi/2;
 }
 
+Real LinfNorm(const ChebyCoeff& f) {
+  fieldstate s = f.state();
+  ((ChebyCoeff&) f).makePhysical();
+
+  Real rtn=0.0;
+  for (int m=0; m<f.N(); ++m) 
+    rtn = Greater(abs(f[m]), rtn);
+      
+  ((ChebyCoeff&) f).makeState(s);
+
+  return rtn;
+}
+
+Real LinfDist(const ChebyCoeff& f, const ChebyCoeff& g) {
+  assert(f.congruent(g));
+  fieldstate fs = f.state();
+  fieldstate gs = f.state();
+  ((ChebyCoeff&) f).makePhysical();
+  ((ChebyCoeff&) g).makePhysical();
+
+  Real rtn=0.0;
+  for (int m=0; m<f.N(); ++m)
+    rtn = Greater(rtn, abs(f[m]-g[m]));
+
+  ((ChebyCoeff&) f).makeState(fs);
+  ((ChebyCoeff&) g).makeState(gs);
+  return rtn;
+}
+
+// Probably INEFFICIENT
+Real L1Norm(const ChebyCoeff& f, bool normalize) {
+  ChebyCoeff g(f);
+  g.makePhysical();
+  for (int n=0; n<g.N(); ++n)
+    g[n] = abs(g[n]);
+  g.makeSpectral();
+
+  ChebyCoeff G(g);
+  integrate(g,G);
+
+  Real rtn = G.eval_b() - G.eval_a();
+
+  if (normalize) 
+    rtn /= g.b() - g.a();
+
+  return rtn;
+}
+
+Real L1Dist(const ChebyCoeff& f, const ChebyCoeff& g, bool normalize) {
+  assert(f.congruent(g));
+  fieldstate fs = f.state();
+  fieldstate gs = f.state();
+  ((ChebyCoeff&) f).makePhysical();
+  ((ChebyCoeff&) g).makePhysical();
+
+  ChebyCoeff d(f);
+  for (int n=0; n<g.N(); ++n)
+    d[n] = abs(f[n]-g[n]);
+  d.makeSpectral();
+
+  ChebyCoeff D(g);
+  integrate(d,D);
+
+  Real rtn = D.eval_b() - D.eval_a();
+
+  if (normalize) 
+    rtn /= d.b() - d.a();
+
+  ((ChebyCoeff&) f).makeState(fs);
+  ((ChebyCoeff&) g).makeState(gs);
+
+  return rtn;
+}
+
 // ===========================================================
 // switchable norms
 Real norm2(const ChebyCoeff& u, NormType n, bool normalize) {
@@ -996,29 +1064,24 @@ void ComplexChebyCoeff::conjugate() {
   im *= -1.0;
 }
 
-void ComplexChebyCoeff::save(const string& filebase) const {
+void ComplexChebyCoeff::save(const string& filebase, fieldstate savestate) const {
+  fieldstate origstate = re.state();
+  ((ChebyCoeff&) *this).makeState(savestate);
+
   string filename(filebase);
   filename += string(".asc");
   ofstream os(filename.c_str());
 
-  os << setprecision(REAL_DIGITS);
+  os << scientific << setprecision(REAL_DIGITS);
   os << "% " << re.length() << ' ' << re.a() << ' ' << re.b() << ' ' 
      << re.state() << '\n';
   for (int n=0; n<re.length(); ++n) 
-    os << re[n] << ' ' << im[n] << endl;
+    os  << setw(REAL_IOWIDTH) << re[n] << ' '  
+	<< setw(REAL_IOWIDTH) << im[n] << '\n';
   os.close();
+
+  ((ChebyCoeff&) *this).makeState(origstate);
 }
-/*****************************
-void ComplexChebyCoeff::m4save(const string& filebase) const {
-  string filename(filebase);
-  filename += string(".asc");
-  ofstream os(filename.c_str());
-  os << setprecision(REAL_DIGITS);
-  for (int n=0; n<re.length(); ++n) 
-    os << re[n] << ' ' << im[n] << endl;
-  os.close();
-}
-******************************/
 
 void ComplexChebyCoeff::binaryDump(ostream& os) const {
   re.binaryDump(os);
@@ -1141,7 +1204,9 @@ ChebyTransform::ChebyTransform(int N, int flags)
     cos_table_(new Real[N-1]),
     q_(new Real[N-1]),
     Q_(new fftw_complex[(N-1)/2 +1]),
-    realfft_plan_(0){
+    flags_(flags),
+    realfft_plan_(0)
+{
   assert(N_ >0);
   assert(N_%2 == 1);
 
@@ -1157,6 +1222,64 @@ ChebyTransform::ChebyTransform(int N, int flags)
   realfft_plan_  = rfftwnd_create_plan(rank, n, FFTW_REAL_TO_COMPLEX, flags);
 }
 
+ChebyTransform::ChebyTransform(const ChebyTransform& t) 
+  : 
+  N_(),
+  sin_table_(new Real[N_-1]),
+  cos_table_(new Real[N_-1]),
+  q_(new Real[N_-1]),
+  Q_(new fftw_complex[(N_-1)/2 +1]),
+  flags_(t.flags_),
+  realfft_plan_(0) 
+{
+  assert(N_ >0);
+  assert(N_%2 == 1);
+
+  int Nb = N_-1;
+  Real piNb = pi/Nb;
+  for (int j=0; j<Nb; ++j) {
+    sin_table_[j] = sin(j*piNb);
+    cos_table_[j] = cos(j*piNb);
+  }
+  int rank=1;
+  int n[1];
+  n[0] = Nb;
+  realfft_plan_  = rfftwnd_create_plan(rank, n, FFTW_REAL_TO_COMPLEX, flags_);
+}
+
+ChebyTransform& ChebyTransform::operator=(const ChebyTransform& t) {
+  if (&t == this)
+    return *this;
+
+  delete[] sin_table_;
+  delete[] cos_table_;
+  delete[] q_;
+  delete[] Q_;
+  rfftwnd_destroy_plan(realfft_plan_);
+
+  N_ = t.N_;
+  sin_table_ = new Real[N_-1];
+  cos_table_ = new Real[N_-1];
+  q_ = new Real[N_-1];
+  Q_ = new fftw_complex[(N_-1)/2 +1];
+  flags_ = t.flags_;
+
+  assert(N_ >0);
+  assert(N_%2 == 1);
+
+  int Nb = N_-1;
+  Real piNb = pi/Nb;
+  for (int j=0; j<Nb; ++j) {
+    sin_table_[j] = sin(j*piNb);
+    cos_table_[j] = cos(j*piNb);
+  }
+  int rank=1;
+  int n[1];
+  n[0] = Nb;
+  realfft_plan_ = rfftwnd_create_plan(rank, n, FFTW_REAL_TO_COMPLEX, flags_);
+  return *this;
+}
+
 ChebyTransform::~ChebyTransform() {
   delete[] sin_table_;
   delete[] cos_table_;
@@ -1330,12 +1453,6 @@ ostream& operator<<(ostream& os, const ComplexChebyCoeff& u) {
   return os;
 }
 
-//Real L1Norm(const ComplexChebyCoeff& u) {return L1Norm(u.re) + L1Norm(u.im);}
-//Real L1Dist(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v) {
-//return L1Dist(u.re, v.re) + L1Dist(u.im, v.im);
-//}
-
-
 // =======================================================================
 // L2 norms
 Real L2Norm2(const ComplexChebyCoeff& u, bool normalize) {
@@ -1399,6 +1516,24 @@ Complex L2InnerProduct(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
 }
 
 
+Real L1Norm(const ComplexChebyCoeff& u, bool normalize) {
+  return L1Norm(u.re, normalize) + L1Norm(u.im, normalize);
+}
+
+Real L1Dist(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v, 
+	    bool normalize) {
+  return L1Dist(u.re, v.re, normalize) + L1Dist(u.im, v.im, normalize);
+}
+
+Real LinfNorm(const ComplexChebyCoeff& u) {
+  return LinfNorm(u.re) + LinfNorm(u.im);
+}
+
+Real LinfDist(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v) {
+  return LinfDist(u.re, v.re) + LinfDist(u.im, v.im);
+}
+
+
 // =======================================================================
 // cheby norms 
 Real chebyNorm2(const ComplexChebyCoeff& u, bool normalize) {
