@@ -1,13 +1,15 @@
 /* chebyshev.h: Real- and Complex-valued Chebyshev expansion classes.
  * Channelflow-0.9
  *
- * Copyright (C) 2001  John F. Gibson  
+ * Copyright (C) 2001-05  John F. Gibson  
  *  
- * jgibson@mail.sjcsf.edu  
  * John F. Gibson 
- * St. John's College
- * 1160 Camino de la Cruz Blanca
- * Santa Fe, NM 87501
+ * Center for Nonlinear Sciences
+ * School of Physics
+ * Georgia Institute of Technology
+ * Atlanta, GA  GA 30332-0430
+ *
+ * gibson@cns.physics.gatech.edu, jfg@member.fsf.org
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,7 +22,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, U
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  */
 
 #ifndef CHEBYSHEV_H
@@ -56,14 +58,13 @@ class ChebyCoeff : public Vector {
 public:
 
   ChebyCoeff();   /// Null constructor
-  ChebyCoeff(int N, Real a, Real b, fieldstate s);
-  ChebyCoeff(const Vector& v, Real a, Real b, fieldstate s);
+  ChebyCoeff(int N, Real a, Real b, fieldstate s=Spectral);
+  ChebyCoeff(const Vector& v, Real a, Real b, fieldstate s=Spectral);
   ChebyCoeff(int N, const ChebyCoeff& g);   // copy first N elems.
   ChebyCoeff(const string& filebase);       // read ascii from file
   ~ChebyCoeff();
 
-  void save(const string& filebase) const;
-  //void m4save(const string& filebase) const;
+  void save(const string& filebase, fieldstate s=Physical) const;
   void binaryDump(ostream& os) const;
   void binaryLoad(istream& is);
 
@@ -110,6 +111,8 @@ public:
 
   bool congruent(const ChebyCoeff& g) const;
 
+
+  friend void swap(ChebyCoeff& f, ChebyCoeff& g);
 private:
   Real a_;             // lower bound of domain
   Real b_;             // upper bound of domain
@@ -121,11 +124,13 @@ ChebyCoeff operator+(const ChebyCoeff& f, const ChebyCoeff& g);
 ChebyCoeff operator-(const ChebyCoeff& f, const ChebyCoeff& g);
 bool operator==(const ChebyCoeff& f, const ChebyCoeff& g);
 
-void diff(const ChebyCoeff& f, ChebyCoeff& df);
+void  diff(const ChebyCoeff& f, ChebyCoeff& df);
 void diff2(const ChebyCoeff& f, ChebyCoeff& d2f);
 void diff2(const ChebyCoeff& f, ChebyCoeff& d2f, ChebyCoeff& tmp);
-ChebyCoeff diff(const ChebyCoeff& f);
+void  diff(const ChebyCoeff& f, ChebyCoeff& df, int n);
+ChebyCoeff  diff(const ChebyCoeff& f);
 ChebyCoeff diff2(const ChebyCoeff& f);
+ChebyCoeff  diff(const ChebyCoeff& f, int n);
 
 // Integrate sets the arbitrary const of integration so that mean(u)==0.
 void integrate(const ChebyCoeff& df, ChebyCoeff& f);
@@ -141,6 +146,7 @@ ChebyCoeff chebyshev(int N, int n, bool normalize=false);
 // L2Dist(f,g)  = sqrt(Int_a^b (f-g)^2 dy)  sqrt(1/(b-a) Int_a^b (f-g)^2 dy
 // L2IP...(f,g) =      Int_a^b f g dy            1/(b-a) Int_a^b f g dy 
 //                               
+
 Real L2Norm2(const ChebyCoeff& f, bool normalize=true); 
 Real L2Dist2(const ChebyCoeff& f, const ChebyCoeff& g, bool normalize=true); 
 Real L2Norm(const ChebyCoeff& f, bool normalize=true); 
@@ -164,15 +170,19 @@ Real dist(const ChebyCoeff& f, const ChebyCoeff& g, NormType n,
 Real innerProduct(const ChebyCoeff& f, const ChebyCoeff& g, NormType n, 
 		    bool normalize=true);
 
-//Real L1Norm(const ChebyCoeff& f, bool normalize=true);
-//Real L1Dist(const ChebyCoeff& f, const ChebyCoeff& g, bool normalize=true);
+Real L1Norm(const ChebyCoeff& f, bool normalize=true);
+Real L1Dist(const ChebyCoeff& f, const ChebyCoeff& g, bool normalize=true);
+
+Real LinfNorm(const ChebyCoeff& f); 
+Real LinfDist(const ChebyCoeff& f, const ChebyCoeff& g); 
+
 
 class ComplexChebyCoeff {
 public:
   ComplexChebyCoeff();
   ComplexChebyCoeff(int N, Real a, Real b, fieldstate s);
   ComplexChebyCoeff(int N, const ComplexChebyCoeff& f); // copy first N elems.
-  ComplexChebyCoeff(const ChebyCoeff& r, const ChebyCoeff& i);
+  ComplexChebyCoeff(const ChebyCoeff& re, const ChebyCoeff& im);
   ComplexChebyCoeff(const string& filename); // read ascii from file
   ComplexChebyCoeff(istream& is);          // read binary from stream
 
@@ -200,10 +210,10 @@ public:
   inline int numModes() const;
   inline fieldstate state() const;
 
-  inline Complex operator[](int i) const;
-  inline void set(int i, Complex c);
-  inline void add(int i, Complex c);
-  inline void sub(int i, Complex c);
+  inline Complex operator[](int n) const;
+  inline void set(int n, Complex c);
+  inline void add(int n, Complex c);
+  inline void sub(int n, Complex c);
 
   ComplexChebyCoeff& operator += (const ComplexChebyCoeff& f);
   ComplexChebyCoeff& operator -= (const ComplexChebyCoeff& f);
@@ -212,8 +222,7 @@ public:
   ComplexChebyCoeff& operator *= (const ComplexChebyCoeff& c); // dottimes
 
   void conjugate(); // destructive
-  void save(const string& filebase) const;
-  void m4save(const string& filebase) const;
+  void save(const string& filebase, fieldstate s=Physical) const;
   void binaryDump(ostream& os) const;
   void binaryLoad(istream& is);
 
@@ -231,6 +240,8 @@ public:
   inline void makePhysical(const ChebyTransform& t);
   inline void makeState(fieldstate s, const ChebyTransform& t);
 
+  friend void swap(ComplexChebyCoeff& f, ComplexChebyCoeff& g);
+
   ChebyCoeff re;
   ChebyCoeff im;
 };
@@ -240,11 +251,16 @@ ComplexChebyCoeff operator+(const ComplexChebyCoeff& f, const ComplexChebyCoeff&
 ComplexChebyCoeff operator-(const ComplexChebyCoeff& f, const ComplexChebyCoeff& g);
 bool operator==(const ComplexChebyCoeff& f, const ComplexChebyCoeff& g);
 
-void diff(const ComplexChebyCoeff& f, ComplexChebyCoeff& df);
+void  diff(const ComplexChebyCoeff& f, ComplexChebyCoeff& df);
 void diff2(const ComplexChebyCoeff& f, ComplexChebyCoeff& d2f);
-void diff2(const ComplexChebyCoeff& f,  ComplexChebyCoeff& d2f, ComplexChebyCoeff& tmp);
-ComplexChebyCoeff diff(const ComplexChebyCoeff& f);
+void diff2(const ComplexChebyCoeff& f, ComplexChebyCoeff& d2f, 
+	   ComplexChebyCoeff& tmp);
+void  diff(const ComplexChebyCoeff& f, ComplexChebyCoeff& d2f, int n);
+
+ComplexChebyCoeff  diff(const ComplexChebyCoeff& f);
 ComplexChebyCoeff diff2(const ComplexChebyCoeff& f);
+ComplexChebyCoeff  diff(const ComplexChebyCoeff& f, int n);
+
 void integrate(const ComplexChebyCoeff& df, ComplexChebyCoeff& f);
 ComplexChebyCoeff integrate(const ComplexChebyCoeff& df);
 
@@ -295,7 +311,11 @@ Real dist(const ComplexChebyCoeff& f, const ComplexChebyCoeff& g, NormType n,
 Complex innerProduct(const ComplexChebyCoeff& f, const ComplexChebyCoeff& g, 
 		  NormType n,  bool normalize=true);
 
+Real L1Norm(const ComplexChebyCoeff& f, bool normalize=true);
+Real L1Dist(const ComplexChebyCoeff& f, const ComplexChebyCoeff& g, bool normalize=true);
 
+Real LinfNorm(const ComplexChebyCoeff& f); 
+Real LinfDist(const ComplexChebyCoeff& f, const ComplexChebyCoeff& g); 
 
 ComplexChebyCoeff chebyEval(const ComplexChebyCoeff& c, const Vector& x);
 
@@ -322,13 +342,16 @@ public:
   void save(const ChebyCoeff& f, const string& name, int deriv=0) const;
   void save(const ComplexChebyCoeff& f, const string& name, int deriv=0) const;
 
+  void call_fftw() const; // for profiling
+
 private:
   int N_; 
-  Real* sin_table_; // sin_table[j] = sin(j*pi/(Ny-1));
-  Real* cos_table_; // cos_table[j] = cos(j*pi/(Ny-1));
+  Real* sin_table_; // sin_table[j] = sin(j*pi/(N-1));
+  Real* cos_table_; // cos_table[j] = cos(j*pi/(N-1));
   Real* q_;         // scratch space for y transforms. Q = Realfftw(q)
   fftw_complex* Q_; // q and Q are meaningless names for temp variables.
-
+  
+  int flags_;
   rfftwnd_plan realfft_plan_;
 };
 
