@@ -1,13 +1,16 @@
 /* poissonsolver.cpp: solve possion equation 
  * Channelflow-0.9
  *
- * Copyright (C) 2005  John F. Gibson  
- *  
- * jgibson@mail.sjcsf.edu  
- * John F. Gibson 
- * St. John's College
- * 1160 Camino de la Cruz Blanca
- * Santa Fe, NM 87501
+ * Copyright (C) 2001-2005  John F. Gibson
+ *
+ * John F. Gibson
+ * Center for Nonlinear Sciences
+ * School of Physics
+ * Georgia Institute of Technology
+ * Atlanta, GA 30332-0430
+ *
+ * gibson@cns.physics.gatech.edu
+ * jfg@member.fsf.org
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -68,6 +71,30 @@ PoissonSolver::PoissonSolver(int Nx, int Ny, int Nz, int Nd, Real Lx, Real Lz,
   }
 }
 
+PoissonSolver::PoissonSolver(const PoissonSolver& ps) 
+  :
+  Mx_(ps.Mx_),
+  My_(ps.My_),
+  Mz_(ps.Mz_),
+  Nd_(ps.Nd_),
+  Lx_(ps.Lx_),
+  Lz_(ps.Lz_),
+  a_(ps.a_),
+  b_(ps.b_),
+  helmholtz_(0) 
+{
+
+  // Allocate Mx x Mz array of HelmholtzSolvers
+  helmholtz_ = new HelmholtzSolver*[Mx_]; // new #1
+  for (int mx=0; mx<Mx_; ++mx)
+    helmholtz_[mx] = new HelmholtzSolver[Mz_];  // new #2
+  
+  // Assign Helmholtz solvers into array. Each solves p'' - lambda p = f
+  for (int mx=0; mx<Mx_; ++mx) 
+    for (int mz=0; mz<Mz_; ++mz) 
+      helmholtz_[mx][mz] = ps.helmholtz_[mx][mz];
+}
+
 PoissonSolver::PoissonSolver(const FlowField& u) 
   :
   Mx_(u.Mx()),
@@ -234,6 +261,24 @@ PressureSolver::PressureSolver()
   nu_(0)
 {}
 
+
+PressureSolver::PressureSolver(int Nx, int Ny, int Nz, Real Lx, Real Lz,
+			       Real a, Real b, const ChebyCoeff& U, Real nu, 
+			       NonlinearMethod nonl)
+  :
+  PoissonSolver(Nx, Ny, Nz, 1, Lx, Lz, a, b),
+  U_(U),
+  trans_(Ny),
+  nonl_(Nx, Ny, Nz, 3, Lx, Lz, a, b),
+  tmp_(), // geom will be set in call to navierstokesNL 
+  div_nonl_(Nx, Ny, Nz, 1, Lx, Lz, a, b),
+  nu_(nu),
+  nonl_method_(nonl)
+{
+  assert(U_.N() == Ny);
+  U_.makeSpectral(trans_);
+}
+
 PressureSolver::PressureSolver(const FlowField& u, Real nu, NonlinearMethod nl)
   :
   PoissonSolver(u.Nx(), u.Ny(), u.Nz(), 1, u.Lx(), u.Lz(), u.a(), u.b()),
