@@ -1,13 +1,15 @@
 /* tausolver.cpp: solves vector "tau" eqns (Canuto & Hussaini eqn 7.3.18-20)
  * Channelflow-0.9
  *
- * Copyright (C) 2001  John F. Gibson  
+ * Copyright (C) 2001-2005  John F. Gibson  
  *  
- * jgibson@mail.sjcsf.edu  
- * John F. Gibson 
- * St. John's College
- * 1160 Camino de la Cruz Blanca
- * Santa Fe, NM 87501
+ * gibson@cns.physics.gatech.edu  jfg@member.fsf.org
+ *
+ * Center for Nonlinear Science
+ * School of Physics
+ * Georgia Institute of Technology
+ * Atlanta, GA 30332-0430
+ * 404 385 2509 
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -56,7 +58,7 @@ Real divcheck(string& label, int kx, int kz, Real kxLx, Real kzLz,
   ComplexChebyCoeff tmp = diff(v);
   Real diverge=0.0;
   for (int n=N-1; n>=0; --n) 
-    diverge += norm2(tmp[n] + (pi*2*(kxLx*u[n] + kzLz*w[n]))*I);
+    diverge += abs2(tmp[n] + (pi*2*(kxLx*u[n] + kzLz*w[n]))*I);
   diverge = sqrt(diverge);
 
   if (diverge > 1e-13 || verbose) {
@@ -77,8 +79,9 @@ TauSolver::TauSolver()
   Nb_(0),
   kx_(0),
   kz_(0),
-  kxLx_(0),
-  kzLz_(0),
+  two_pi_kxLx_(0),
+  two_pi_kzLz_(0),
+  kappa2_(0),
   a_(0),
   b_(0),
   lambda_(0),
@@ -105,14 +108,15 @@ TauSolver::TauSolver(int kx, int kz, Real Lx, Real Lz, Real a, Real b,
   Nb_(nChebyModes-1),
   kx_(kx),
   kz_(kz),
-  kxLx_(Real(kx)/Lx),
-  kzLz_(Real(kz)/Lz),
+  two_pi_kxLx_(2*pi*kx/Lx),
+  two_pi_kzLz_(2*pi*kz/Lz),
+  kappa2_(4*square(pi)*(square(kx/Lx)+square(kz/Lz))),
   a_(a),
   b_(b),
   lambda_(lambda),
   nu_(nu),
   tauCorrection_(tauCorrection),
-  pressureHelmholtz_(N_, a_, b_, 4*pi*pi*(kxLx_*kxLx_ + kzLz_*kzLz_)),
+  pressureHelmholtz_(N_, a_, b_, kappa2_),
   velocityHelmholtz_(N_, a_, b_, lambda_, nu_),
   P_0_(N_, a_, b_, Spectral),
   v_0_(N_, a_, b_, Spectral),
@@ -125,6 +129,7 @@ TauSolver::TauSolver(int kx, int kz, Real Lx, Real Lz, Real a, Real b,
   i10_(0),
   i11_(0)
 {
+  
   // ======================================================================
   // Calculate the influence matrix.
 
@@ -157,7 +162,7 @@ TauSolver::TauSolver(int kx, int kz, Real Lx, Real Lz, Real a, Real b,
 
   // We know influence matrix is rank-deficient for kx==kz==0. It shouldn't
   // be for other wave numbers.
-  if (kxLx_ !=0 || kzLz_!=0) 
+  if (kx_ !=0 || kz_!=0) 
     assert((abs(discriminant) / Greater(abs(A*D), abs(B*C))) > MINIMUM_DISCRIMINANT);
 
   // Go ahead and divide by zero for kx==kz==0 case. Influence matrix is
@@ -284,7 +289,7 @@ Real TauSolver::verify_P_and_v(const ChebyCoeff& P, const ChebyCoeff& v,
   ChebyCoeff Py =  diff(P);
   ChebyCoeff Pyy = diff(Py);
   ChebyCoeff p_lhs(P);
-  p_lhs *= - 4*pi*pi*(kxLx_*kxLx_ + kzLz_*kzLz_);
+  p_lhs *= -kappa2_;
   p_lhs += Pyy;
 
   Real l2err = L2Dist(p_lhs, r);
@@ -344,7 +349,7 @@ Real TauSolver::verify_P_and_v(const ChebyCoeff& P, const ChebyCoeff& v,
     (abs(nu_*L2Norm(vyy)) + abs(lambda_*L2Norm(v))) + L2Norm(Py);
   v_norm = (v_norm > EPSILON) ? v_norm : 1.0;
   
-  Real p_norm = L2Norm(Pyy) + 4*pi*pi*(kxLx_*kxLx_ + kzLz_*kzLz_)*L2Norm(P);
+  Real p_norm = L2Norm(Pyy) + kappa2_*L2Norm(P);
   p_norm = (p_norm > EPSILON) ? p_norm : 1.0;
 
 
@@ -374,12 +379,10 @@ void TauSolver::solve(ComplexChebyCoeff& u, ComplexChebyCoeff& v,
 		      ComplexChebyCoeff& w, ComplexChebyCoeff& P, 
 		      const ComplexChebyCoeff& Rx, 
 		      const ComplexChebyCoeff& Ry, 
-		      const ComplexChebyCoeff& Rz) 
-  const {
+		      const ComplexChebyCoeff& Rz) const 
+{
 
   ComplexChebyCoeff r(N_,a_,b_,Spectral);
-  Real pi2 = 2*pi;
-  Complex pi2i = pi2*I;
   Real sigmaNb1;
   Real sigmaNb;
   
@@ -390,30 +393,44 @@ void TauSolver::solve(ComplexChebyCoeff& u, ComplexChebyCoeff& v,
   diff(Ry.re, rr);
   int n; // MSVC++ FOR-SCOPE BUG
   for (n=0; n<N_; ++n) 
-    rr[n] -= pi2*(kxLx_*Rx.im[n] + kzLz_*Rz.im[n]); 
+    rr[n] -= two_pi_kxLx_*Rx.im[n] + two_pi_kzLz_*Rz.im[n]; 
   solve_P_and_v(P.re, v.re, rr, Ry.re, sigmaNb1, sigmaNb);
 
   diff(Ry.im, rr);
   for (n=0; n<N_; ++n) 
-    rr[n] += pi2*(kxLx_*Rx.re[n] + kzLz_*Rz.re[n]);
+    rr[n] += two_pi_kxLx_*Rx.re[n] + two_pi_kzLz_*Rz.re[n];
   solve_P_and_v(P.im, v.im, rr, Ry.im, sigmaNb1, sigmaNb);
 
   // Re and Im parts of u and w eqns seperate. 
   // Use r as temporary space to store RHS of eqns.
-  for (n=0; n<N_; ++n) {
-    Complex c = pi2i*kxLx_*P[n] - Rx[n];
-    r.set(n, c);
-  }
+  for (n=0; n<N_; ++n) 
+    r.set(n, two_pi_kxLx_*I*P[n] - Rx[n]);
+  //Complex c = pi2i*kxLx_*P[n] - Rx[n];
+  //r.set(n,c);
+
   velocityHelmholtz_.solve(u.re, r.re, 0.0, 0.0);
   velocityHelmholtz_.solve(u.im, r.im, 0.0, 0.0);
 
-  for (n=0; n<N_; ++n) {
-    Complex c = pi2i*kzLz_*P[n] - Rz[n];
-    r.set(n, c);
-  }
+  for (n=0; n<N_; ++n) 
+    r.set(n, two_pi_kzLz_*I*P[n] - Rz[n]);
+  //Complex c = pi2i*kzLz_*P[n] - Rz[n];
+  //r.set(n, c);
+
   velocityHelmholtz_.solve(w.re, r.re, 0.0, 0.0);
   velocityHelmholtz_.solve(w.im, r.im, 0.0, 0.0);
-    
+
+  // This is for debugging ONLY
+  /**************************
+  if ((two_pi_kxLx_ == 0.0 & kx_ != 0) ||
+      (two_pi_kzLz_ == 0.0 & kz_ != 0)) {
+    verify(u,v,w,P,Rx,Ry,Rz,true);
+    u.setToZero();
+    v.setToZero();
+    w.setToZero();
+    P.setToZero();
+  }
+  **************************/
+	     
 #ifdef DEBUG
   //verify(u,v,w,P,Rx,Ry,Rz);
 #endif
@@ -445,27 +462,25 @@ void TauSolver::solve(ComplexChebyCoeff& u, ComplexChebyCoeff& v,
   diff(Ry.re, rr);
   int n; // MSVC++ FOR-SCOPE BUG
   for (n=0; n<N_; ++n) 
-    rr[n] -= pi2*(kxLx_*Rx.im[n] + kzLz_*Rz.im[n]); 
+    rr[n] -= two_pi_kxLx_*Rx.im[n] + two_pi_kzLz_*Rz.im[n]; 
   solve_P_and_v(P.re, v.re, rr, Ry.re, sigmaNb1, sigmaNb);
 
   diff(Ry.im, rr);
   for (n=0; n<N_; ++n) 
-    rr[n] += pi2*(kxLx_*Rx.re[n] + kzLz_*Rz.re[n]);
+    rr[n] += two_pi_kxLx_*Rx.re[n] + two_pi_kzLz_*Rz.re[n];
   solve_P_and_v(P.im, v.im, rr, Ry.im, sigmaNb1, sigmaNb);
 
   // Re and Im parts of u and w eqns seperate. 
   // Use r as temporary space to store RHS of eqns.
-  for (n=0; n<N_; ++n) {
-    Complex c = pi2i*kxLx_*P[n] - Rx[n];
-    r.set(n, c);
-  }
+  for (n=0; n<N_; ++n) 
+    r.set(n, two_pi_kxLx_*I*P[n] - Rx[n]);
+
   velocityHelmholtz_.solve(u.re, dPdx, r.re, umean, 0.0, 0.0);
   velocityHelmholtz_.solve(u.im, r.im, 0.0, 0.0);
   
-  for (n=0; n<N_; ++n) {
-    Complex c = pi2i*kzLz_*P[n] - Rz[n];
-    r.set(n, c);
-  }
+  for (n=0; n<N_; ++n) 
+    r.set(n, two_pi_kzLz_*I*P[n] - Rz[n]);
+
   velocityHelmholtz_.solve(w.re, r.re, 0.0, 0.0);
   velocityHelmholtz_.solve(w.im, r.im, 0.0, 0.0);
 
@@ -494,14 +509,13 @@ Real TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
 		       const ComplexChebyCoeff& Rz, 
 		       Real umean, bool verbose) const {
 
-  Real pi2 = 2*pi;
   // verify   nu u''(y) - lambda u(y) - grad P = -R, 
   //                                     div u = 0
   //                                    u(+-1) = 0
 
   if (verbose) {
     cerr << "TauSolver::verify(u,v,w,P,dPdx,Rx,Ry,Rz,umean,verbose) {" << endl;
-    cerr << " kx kz == " << pi2*kxLx_ << ' ' << pi2*kzLz_ << endl;
+    cerr << " kx kz == " << kx_ << ' ' << kz_ << endl;
   }
   ComplexChebyCoeff lhs(N_,a_,b_,Spectral);
   ComplexChebyCoeff tmp(N_,a_,b_,Spectral);
@@ -517,7 +531,7 @@ Real TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
   lhs -= tmp;
   tmp = P;
   //tmp *= pi2*I*kxLx_; gcc-2.95 bug makes the following necessary
-  tmp *= I*pi2*kxLx_;
+  tmp *= Complex(0.0, two_pi_kxLx_);
   lhs += tmp;
   lhs.re[0] += dPdx;
 
@@ -554,7 +568,7 @@ Real TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
   tmp *= nu_;
   lhs -= tmp;
   tmp = P;
-  tmp *= I*pi2*kzLz_;
+  tmp *= Complex(0.0, two_pi_kzLz_);
   lhs += tmp;
   terr = tauDist(lhs, Rz);
   lerr = L2Dist(lhs, Rz);
@@ -568,7 +582,7 @@ Real TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
   // Verify P eqn. P'' - kappa^2 P = div R
   diff2(P, lhs);
   tmp = P;
-  tmp *= - 4*pi*pi*(kxLx_*kxLx_ + kzLz_*kzLz_);
+  tmp *= -kappa2_;
   lhs += tmp;
 
   ComplexChebyCoeff r(N_,a_,b_,Spectral);
@@ -579,11 +593,11 @@ Real TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
   diff(Ry.re, r_re);
   int n; // MSVC++ FOR-SCOPE BUG
   for (n=0; n<N_; ++n) 
-    r_re[n] -= pi2*(kxLx_*Rx.im[n] + kzLz_*Rz.im[n]); 
+    r_re[n] -= two_pi_kxLx_*Rx.im[n] + two_pi_kzLz_*Rz.im[n]; 
 
   diff(Ry.im, r_im);
   for (n=0; n<N_; ++n) 
-    r_im[n] += pi2*(kxLx_*Rx.re[n] + kzLz_*Rz.re[n]);
+    r_im[n] += two_pi_kxLx_*Rx.re[n] + two_pi_kzLz_*Rz.re[n];
   
   r.re = r_re;
   r.im = r_im;
@@ -591,7 +605,6 @@ Real TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
   lerr = L2Dist(lhs, r);
   error += lerr;
   if (verbose) {
-    cerr << "really ..." << endl;
     cerr << "L2Norm(div R) == " << L2Norm(r) << endl;
     cerr << "tauDist(P'' - k^2 P, div R)   == " << terr << endl;
     cerr << " L2Dist(P'' - k^2 P, div R)   == " << lerr << endl;
@@ -604,7 +617,8 @@ Real TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
   // Verify divergence
   diff(v, tmp);
   for (n=0; n<N_; ++n) 
-    tmp.add(n, I*pi2*(kxLx_*u[n] + kzLz_*w[n]));
+    tmp.add(n, I*(two_pi_kxLx_*u[n] + two_pi_kzLz_*w[n]));
+    //tmp.add(n, I*pi2*(kxLx_*u[n] + kzLz_*w[n]));
 
   terr = tauNorm(tmp);
   lerr = L2Norm(tmp);
@@ -641,10 +655,10 @@ Real TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
   if (verbose) 
     cerr << "w(a),w(b) == " << wa << " " << wb << endl;
 
-  Real mean_error = norm2(Re(u.mean()) - umean);
+  Real mean_error = abs2(Re(u.mean()) - umean);
   error += mean_error;
   if (verbose)
-    cerr << "norm2(u.mean() - umean) == " << mean_error << endl;
+    cerr << "abs2(u.mean() - umean) == " << mean_error << endl;
   else
     assert(mean_error < EPSILON);
   
