@@ -1,13 +1,15 @@
 /* tausolver.h: solves vector "tau" eqns (Canuto & Hussaini eqn 7.3.18-20)
  * Channelflow-0.9
  *
- * Copyright (C) 2001  John F. Gibson  
+ * Copyright (C) 2001-2005  John F. Gibson  
  *  
- * jgibson@mail.sjcsf.edu  
- * John F. Gibson 
- * St. John's College
- * 1160 Camino de la Cruz Blanca
- * Santa Fe, NM 87501
+ * gibson@cns.physics.gatech.edu  jfg@member.fsf.org
+ *
+ * Center for Nonlinear Science
+ * School of Physics
+ * Georgia Institute of Technology
+ * Atlanta, GA 30332-0430
+ * 404 385 2509 
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,7 +33,6 @@
 #include "cfvector.h"
 #include "chebyshev.h"
 
-
 // Class for solving 7.3.18-7.3.20 of Canuto & Hussaini
 //      nu u''_jk(y) - lambda u_jk(y) - grad P_jk = -R_jk, 
 //                                       div u_jk = 0
@@ -41,17 +42,18 @@
 //          P(y) is in R
 // and the vector operators are interpreted accordingingly (diff in x 
 // equal multiplication by 2 pi k).
-
 Real divcheck(string& label, int kx, int kz, Real Lx, Real Lz, 
 	      const ComplexChebyCoeff& u, const ComplexChebyCoeff& v, 
 	      const ComplexChebyCoeff& w, bool verbose=false);
 
-
 class TauSolver {
 public:
   TauSolver();  
-  TauSolver(int kx, int kz, Real Lx, Real Lz, Real a, Real b,
-	    Real lambda, Real nu, int nChebyModes, bool tauCorrection=true);
+  TauSolver(int kx, int kz, Real Lx, Real Lz, Real a, Real b, Real lambda, 
+	    Real nu, int nChebyModes, bool tauCorrection=true);
+  //TauSolver(int kx, int kz, Real Lx, Real Lz, Real a, Real b, Real lambda, 
+  //Real nu, int nChebyModes, bool dx_on=true, bool dz_on=true,
+  //bool tauCorrection=true);
 
   void solve(ComplexChebyCoeff& u, ComplexChebyCoeff& v, ComplexChebyCoeff& w,
 	     ComplexChebyCoeff& P, const ComplexChebyCoeff& Rx, 
@@ -91,18 +93,19 @@ public:
 
   int kx() const {return kx_;}
   int kz() const {return kz_;}
-  Real kxLx() const{return kxLx_;}
-  Real kzLz() const{return kzLz_;}
+  //Real kxLx() const{return kxLx_;}
+  //Real kzLz() const{return kzLz_;}
   Real lambda() const{return lambda_;}
   Real nu() const{return nu_;}
 
 private:
   int N_;        // number of Chebyshev modes
   int Nb_;       // N-1. Convenience variable, corresponds to C&H's N.
-  int kx_;
-  int kz_;
-  Real kxLx_;
-  Real kzLz_;
+  int kx_;       // x wave number
+  int kz_;       // z wave number
+  Real two_pi_kxLx_; // 2 pi kx/Lx
+  Real two_pi_kzLz_; // 2 pi kz/Lz
+  Real kappa2_;  // 4 pi^2 [(kx/Lx)^2 + (kz/Lz)^2]
   Real a_;
   Real b_;
   Real lambda_;  
