@@ -28,13 +28,13 @@ int main() {
 
   // Define integration parameters
   const int n = 10;       // take n steps between printouts
-  const Real dt = 0.1;
+  const Real dt = 0.01;
   const Real T0  = 0.0;
   const Real T1  = 10.0;
 
   // Define size and smoothness of initial disturbance
   Real decay = 0.6;   
-  Real magnitude  = 0.1;   
+  Real magnitude  = 0.01;   
   int kxmax = 3;
   int kzmax = 3;
 
@@ -74,9 +74,6 @@ int main() {
   NSIntegrator dns_skw(u_skw, U, nu, dt, flags_skw,T0);
 
   for (Real t=T0; t<T1; t += n*dt) {
-    dns_rot.advance(u_rot, q_rot, n);
-    dns_skw.advance(u_skw, q_skw, n);
-
     ComplexChebyCoeff u12;
     u12 = u_rot.profile(1,2,0);
     u12.makePhysical();
@@ -84,13 +81,18 @@ int main() {
     u12 = u_skw.profile(1,2,0);
     u12.makePhysical();
     u12.save("uskw12");
-
+    
+    cout << "rot dns CFL == " << dns_rot.CFL() << endl;
+    cout << "skw dns CFL == " << dns_skw.CFL() << endl;
     cout << "L2Dist2(u_rot,u_skw) == " << L2Dist2(u_rot,u_skw) << endl;
     cout << "   L2Dist2(u_rot,u0) == " << L2Dist2(u_rot,u0) << endl;
     cout << "   L2Dist2(u_skw,u0) == " << L2Dist2(u_skw,u0) << endl;
     cout << "      L2Norm2(u_rot) == " << L2Norm2(u_rot) << endl;
     cout << "      L2Norm2(u_skw) == " << L2Norm2(u_skw) << endl;
     cout << "         L2Norm2(u0) == " << L2Norm2(u0) << endl;
+
+    dns_rot.advance(u_rot, q_rot, n);
+    dns_skw.advance(u_skw, q_skw, n);
   }
 }
 
