@@ -1,5 +1,6 @@
 #include <iostream>
 #include <iomanip>
+#include <fstream>
 #include "helmholtz.h"
 #include "cfvector.h"
 #include "tausolver.h"
@@ -11,15 +12,20 @@ void randomProfile(ComplexChebyCoeff& u, ComplexChebyCoeff& v,
 		   ComplexChebyCoeff& w, ComplexChebyCoeff& P, 
 		   int kx, int kz, Real Lx, Real Lz, Real decay);
 
+void uniformsave(const ComplexChebyCoeff& u, const string& filebase);
+
+const int Nunif=100;
+  
 int main() {
 
-  bool save = false;
-  bool verbose = false;
+  bool save = true;
+  bool verbose = true;
   
+  bool taucorrect = true;
   int N=133; // Chebyshev expansion length;
-  int nTests=10;
+  int nTests=1;
   Real epsilon =1e-12;
-
+  
   if (verbose) 
     cout << setprecision(14);
   ChebyTransform trans(N);
@@ -28,11 +34,11 @@ int main() {
   for (int test=0; test<nTests; ++test) {
 
     Real Lx = 2*pi*(1 + drand48());
-    Real Lz =  2*pi*(1 + drand48());
-    Real a = drand48();
-    Real b = a + (2 + drand48());
-    int kx = rand() % 32;
-    int kz = rand() % 32;
+    Real Lz = 2*pi*(1 + drand48());
+    Real a = -1; //drand48();
+    Real b = 1; // a + (2 + drand48());
+    int kx = 3; //rand() % 32;
+    int kz = 2; // rand() % 32;
     Real dt = 0.02;
     Real nu =  1.0/1000.0;
     Real lambda = 2.0/dt + 4*pi*pi*nu*(square(kx/Lx) + square(kz/Lz));
@@ -53,8 +59,8 @@ int main() {
 
     // Construct a div-free field (u,v,w) with zero BCS at +/-1 and a 
     // random zero-mean pressure field P. From those, calculate 
-    // R = -(nu del^2 u'' - lambda u - grad P). Then use TauSolver to solve 
-    // nu del^2 u - lambda u - grad P = -R for u and P from R, and compare.
+    // R = (nu del^2 u'' - lambda u - grad P). Then use TauSolver to solve 
+    // nu del^2 u - lambda u - grad P = R for u and P from R, and compare.
 
     ComplexChebyCoeff P(N,a,b,Spectral);
     ComplexChebyCoeff u(N,a,b,Spectral);
@@ -64,7 +70,7 @@ int main() {
     randomProfile(u,v,w,P,kx,kz,Lx,Lz,decay);
 
     ComplexChebyCoeff vy = diff(v);
-    // Calculate R = -nu u'' + lambda u + grad P
+    // Calculate R = nu lapl u'' - lambda u - grad P
     ComplexChebyCoeff Rx(u);
     ComplexChebyCoeff Ry(v);
     ComplexChebyCoeff Rz(w);
@@ -88,8 +94,8 @@ int main() {
 
     ComplexChebyCoeff Px(P);
     ComplexChebyCoeff Pz(P);
-    Px *= 2*pi*I*(kx/Lx);
-    Pz *= 2*pi*I*(kz/Lz);
+    Px *= Complex(0.0, 2*pi*(kx/Lx));
+    Pz *= Complex(0.0, 2*pi*(kz/Lz));
     ComplexChebyCoeff Py(N,a,b,Spectral);
     diff(P,Py);
 
@@ -103,9 +109,11 @@ int main() {
     ComplexChebyCoeff vysolve(N,a,b,Spectral);
     ComplexChebyCoeff wsolve(N,a,b,Spectral);
     ComplexChebyCoeff Psolve(N,a,b,Spectral);
+    Real sigma0 = 0;
+    Real sigma1 = 0;
     if (verbose) 
       cout << "Constructing TauSolver {" << endl;
-    TauSolver tausolver(kx, kz, Lx, Lz, a, b, lambda, nu, N);
+    TauSolver tausolver(kx, kz, Lx, Lz, a, b, lambda, nu, N, taucorrect);
     if (verbose) {
       cout << "} done ctoring TauSolver" << endl;
       cout << "----------------------------------------------------" << endl;
@@ -125,7 +133,7 @@ int main() {
       cout << "----------------------------------------------------" << endl;
       cout << "Verifying numerical Tau solution {" << endl;
     }
-    error += tausolver.verify(usolve, vsolve, wsolve, Psolve, Rx, Ry, Rz, verbose);
+   error += tausolver.verify(usolve, vsolve, wsolve, Psolve, Rx, Ry, Rz, verbose);
     if (verbose) {
       cout << "} done verifying numerical Tau problem" << endl;
       cout << "----------------------------------------------------" << endl;
@@ -173,6 +181,28 @@ int main() {
       trans.save(Rz, "Rz");
       trans.save(diverge, "divs");
 
+      Vector yf(Nunif); 
+      Real dy = (b-a)/Nunif;
+      for (int n=0; n<Nunif; ++n)
+	yf[n] = a + n*dy;
+
+      yf.save("yf");
+      uniformsave(u, "uf");
+      uniformsave(v, "vf");
+      uniformsave(w, "wf");
+      uniformsave(P, "Pf");
+      uniformsave(Py, "Pyf");
+      uniformsave(vy, "vyf");
+      uniformsave(usolve, "usf");
+      uniformsave(vsolve, "vsf");
+      uniformsave(wsolve, "wsf");
+      uniformsave(Psolve, "Psf");
+      uniformsave(vysolve, "vysf");
+      uniformsave(Rx, "Rxf");
+      uniformsave(Ry, "Ryf");
+      uniformsave(Rz, "Rzf");
+      uniformsave(diverge, "divsf");
+
       ComplexChebyCoeff uyy = diff2(u);
       ComplexChebyCoeff vyy = diff2(v);
       ComplexChebyCoeff wyy = diff2(w);
@@ -188,7 +218,9 @@ int main() {
       trans.save(wyys, "wyys");
       trans.save(Pys, "Pys");
       trans.save(vysolve, "vys");
-    
+
+      
+
       trans.ichebyfft(usolve);
       trans.ichebyfft(vsolve);
       trans.ichebyfft(vysolve);
@@ -336,3 +368,18 @@ void randomProfile(ComplexChebyCoeff& u, ComplexChebyCoeff& v, ComplexChebyCoeff
   div += wz;
 }
 
+void uniformsave(const ComplexChebyCoeff& f, const string& filebase) {
+  string filename = filebase + ".asc";
+  ofstream os(filename.c_str());
+
+  ComplexChebyCoeff g(f);
+  g.makeSpectral();
+  Real a = g.a();
+  Real b = g.b();
+  Real dy = (b-a)/(Nunif-1);
+  for (int n=0; n<Nunif; ++n) {
+    Real y = a+n*dy;
+    os << g.re.eval(y) << ' ' << g.im.eval(y) << '\n';
+  }
+  os.close();
+}
